@@ -271,7 +271,9 @@ impl EspMqttClient {
 
         let unsafe_callback = UnsafeCallback::from(&mut boxed_raw_callback);
 
-        let (c_conf, _cstrs) = conf.into();
+        let (mut c_conf, mut cstrs) = conf.into();
+
+        c_conf.uri = cstrs.as_ptr(url);
 
         let client = unsafe { esp_mqtt_client_init(&c_conf as *const _) };
         if client.is_null() {
@@ -280,10 +282,6 @@ impl EspMqttClient {
 
         let client = Self(client, boxed_raw_callback);
 
-        let c_url = CString::new(url.as_ref()).unwrap();
-
-        esp!(unsafe { esp_mqtt_client_set_uri(client.0, c_url.as_ptr()) })?;
-
         esp!(unsafe {
             esp_mqtt_client_register_event(
                 client.0,
@@ -321,6 +319,7 @@ impl EspMqttClient {
 #[cfg(feature = "experimental")]
 impl EspMqttClient {
     pub fn new_async<'a, U, S>(
+        unblocker: U,
         url: S,
         conf: /*TODO: issue with the compiler: &'a*/ MqttClientConfiguration<'a>,
     ) -> Result<
@@ -349,7 +348,7 @@ impl EspMqttClient {
         let client = Self::new_with_callback(url, &conf, move |event| cb_connection.post(event))?;
 
         Ok((
-            embedded_svc::utils::asyncify::mqtt::client::AsyncClient::new(client),
+            embedded_svc::utils::asyncify::mqtt::client::AsyncClient::new(unblocker, client),
             connection,
         ))
     }
